#include <avr/io.h>
#include "interface.h"
#include "monitor.h"
#include "motorola.h"
#include "output.h"
#include "serial.h"
#include "s88.h"

static uint8_t cmd_buffer[15];
static uint8_t cmd_length = 0;
static uint8_t cmd_read_pos = 0;

static uint8_t dispatch_command(const uint8_t *, uint8_t);

void interface_init(void)
{
	DDRD = (DDRD&0xFC)|0x02;

	serial_init(9600);
}

void interface_check(void)
{
	uint8_t count;
	if(serial_read_overrun())
		interface_send1(RECEIVE_OVERRUN);

	count = serial_read_available();
	if(count>0)
	{
		if(cmd_length==0)
		{
			uint8_t l = ~serial_read();
			if(l==0)
				serial_write(0xFF);
			else if(l>=0x10)
				interface_send1(FRAMING_ERROR);
			else
			{
				cmd_length = l;
				--count;
				cmd_read_pos = 0;
			}
		}

		if(cmd_read_pos<cmd_length)
		{
			uint8_t i;
			if(cmd_read_pos+count>cmd_length)
				count = cmd_length-cmd_read_pos;
			for(i=0; i<count; ++i)
				cmd_buffer[cmd_read_pos++] = serial_read();

			if(cmd_read_pos>=cmd_length)
			{
				uint8_t result = dispatch_command(cmd_buffer, cmd_length);
				interface_send1(result);
				cmd_length = 0;
			}
		}
	}
}

static uint8_t dispatch_command(const uint8_t *cmd, uint8_t length)
{
	uint8_t type = cmd[0]>>4;
	if(type==0)
	{
		uint8_t subtype = (cmd[0]>>3)&1;
		if(subtype==0)
			return output_command(cmd, length);
		else
			return monitor_command(cmd, length);
	}
	else if(type==1)
		return motorola_command(cmd, length);
	else if(type==3)
		return s88_command(cmd, length);
	else
		return INVALID_COMMAND;
}

void interface_send(const uint8_t *cmd, uint8_t length)
{
	uint8_t i;

	serial_write(~length);
	for(i=0; i<length; ++i)
		serial_write(cmd[i]);
}

void interface_send1(uint8_t cmd)
{
	serial_write(0xFE);
	serial_write(cmd);
}
